@@ -26,7 +26,7 @@ import shlex
 import subprocess
 import sys
 from xml.etree import ElementTree as ET
-from nmap3.utils import get_nmap_path
+from nmap3.utils import get_nmap_path # from utils import get_nmap_path
 
 class NmapCommandParser(object):
     """
@@ -114,29 +114,30 @@ class NmapCommandParser(object):
         Performs parsing for nmap pingscan xml rests
         @ return DICT
         """
-        ping_status = dict()
+        ping_status_list = []
         try:
             
             if not xml_root:
                 return host_list
             self.xml_root == xml_root
-            host = xml_root.find("host")
             
-            if(host):
-                ping_status = host.find('status').attrib
-                address = []
-                hostname = []
-                
-                for addr in host.findall("address"):
-                    address.append(addr.attrib)
-                ping_status["addresses"]=address
-                
-                if(host.find("hostnames")):
-                    for host_n in host.find("hostnames").findall("hostname"):
-                        hostname.append(host_n.attrib)
-                ping_status["hostname"]=hostname
-                
-            return ping_status
+            for host in xml_root.findall("host"):
+                host_ping_status = dict()
+                if(host):
+                    host_ping_status = host.find('status').attrib
+                    address = []
+                    hostname = []
+                    
+                    for addr in host.findall("address"):
+                        address.append(addr.attrib)
+                    host_ping_status["addresses"]=address
+                    
+                    if(host.find("hostnames")):
+                        for host_n in host.find("hostnames").findall("hostname"):
+                            hostname.append(host_n.attrib)
+                    host_ping_status["hostname"]=hostname
+                ping_status_list.append(host_ping_status)
+            return ping_status_list
         except Exception as e:
             raise
             
@@ -186,7 +187,6 @@ class NmapCommandParser(object):
         except Exception as e:
             raise
         
-        
     def filter_subdomains(self, xmlroot):
         """
         Given the xmlroot return the all the ports that are open from 
@@ -217,15 +217,149 @@ class NmapCommandParser(object):
                     elem = table.findall("elem")
                     
                     if(len(elem) >= 2):
-                        address = elem[0]
-                        hostname = elem[1]
-                        
-                        script_results["address"]=address.text 
-                        script_results["hostname"]=hostname.text 
+                        script_results[elem[0].attrib["key"]] = elem[0].text
+                        script_results[elem[1].attrib["key"]] = elem[1].text
                         subdomains_list.append(script_results)
 
-
         except Exception as e:
             raise(e)
         else:
             return subdomains_list
+            
+    def parse_noportscan(self, xmlroot):
+        """
+        Given the xmlroot return the all the ports that are open from 
+        that tree
+        """
+        
+        result_dicts = {}
+        hosts_list=[]
+        
+        # Find all hosts 
+        all_hosts = xmlroot.findall("host")
+        for host in all_hosts:
+            
+            host_record = {}
+            if(host.find("status") != None):
+                for key in host.find("status").attrib:
+                    host_record[key]=host.find("status").attrib.get(key)
+                
+                for key in host.find("address").attrib:
+                    host_record[key]=host.find("address").attrib.get(key)
+                    
+            hosts_list.append(host_record)
+                
+        runstats = xmlroot.find("runstats")
+        if(runstats):
+            
+            if(runstats.find("finished") != None):
+                result_dicts["runtime"]=runstats.find("finished").attrib
+                result_dicts["status"]=runstats.find("hosts").attrib
+        
+        result_dicts["hosts"]=hosts_list
+        return result_dicts 
+    
+    def filter_top_ports(self, xmlroot):
+        """
+        Given the xmlroot return the all the ports that are open from
+        that tree
+        """
+        try:
+            port_result_dict = {}
+            
+            scanned_host = xmlroot.findall("host")
+            stats = xmlroot.attrib
+            
+            for hosts in scanned_host:
+                address = hosts.find("address").get("addr")
+                
+                ports = hosts.find("ports").findall("port")
+                port_results =[]
+                
+                for port in ports:
+                    open_ports = {}
+                    for key in port.attrib:
+                        open_ports[key]=port.attrib.get(key)
+                        
+                    if port.find('state') != None:
+                        for key in port.find('state').attrib:
+                            open_ports[key]=port.find("state").attrib.get(key)
+
+                    if  port.find("service") != None:
+                        open_ports["service"]=port.find("service").attrib
+
+                    port_results.append(open_ports)
+                port_result_dict[address]=port_results
+                
+            runstats = xmlroot.find("runstats")
+            if(runstats):
+                if(runstats.find("finished") != None):
+                    port_result_dict["runtime"]=runstats.find("finished").attrib
+            port_result_dict["stats"]=stats
+            
+        except Exception as e:
+            raise(e)
+        else:
+            return port_result_dict
+    
+    def version_parser(self, xmlroot):
+        """
+        Parse version detected
+        """
+        service_version = []
+
+        host = xmlroot.find("host")
+        if(host):
+            ports  = host.find("ports")
+            port_service = None
+
+            if(ports):
+                port_service = ports.findall("port")
+
+            if(port_service):
+                for port in port_service:
+                    service = {}
+
+                    service["protocol"]=port.attrib.get("protocol")
+                    service["port"]=port.attrib.get("portid")
+
+                    if(port.find("state")):
+                        for s in port.find("state").attrib:
+                            service[s]=port.find("state").attrib.get(s)
+
+                    if(port.find("service")):
+                        service["service"]=port.find("service").attrib
+
+                        for cp in port.find("service").findall("cpe"):
+                            cpe_list = []
+                            cpe_list.append({"cpe":cp.text})
+                            service["cpe"]=cpe_list
+
+                    service_version.append(service)
+        return service_version
+    
+    def os_identifier_parser(self, xmlroot):
+        """
+        Parser for identified os
+        """
+        try:
+            os_identified = []
+
+            host = xmlroot.find("host")
+            if(host):
+                os = host.find("os")
+
+                if(host):
+                    for match in os.findall("osmatch"):
+                        attrib = match.attrib
+
+                        for osclass in match.findall("osclass"):
+                            attrib["osclass"]=osclass.attrib
+
+                            for cpe in osclass.findall("cpe"):
+                                attrib["cpe"]=cpe.text
+                        os_identified.append(attrib)
+        except Exception as e:
+            raise(e)
+        else:
+            return os_identified
